@@ -1,14 +1,16 @@
 """
 Quandl's API for Python.
 
-Currently supports getting and pushing datasets.
+Currently supports getting, searching, and pushing datasets.
 
 """
 from __future__ import (print_function, division, absolute_import,
                         unicode_literals)
 import pickle
 import datetime
+import json
 import pandas as pd
+import re
 
 from dateutil import parser
 from numpy import genfromtxt
@@ -22,6 +24,7 @@ except ImportError:
     from urllib2 import HTTPError, Request, urlopen
 
 
+
 QUANDL_API_URL = 'http://www.quandl.com/api/v1/'
 
 
@@ -147,9 +150,58 @@ def push(data, code, name, authtoken='', desc='', override=False):
     return rtn
 
 
-# Helper function to parse dates with None fallback
+def search(query, source = None, page= 1 , authtoken = None, prints = True):
+    """Return array of dictionaries of search results.
+
+    :param str query: (required), query to search with
+    :param str source: (optional), source to search
+    :param +'ve int: (optional), page number of search 
+    :param str authotoken: (optional) Quandl auth token for extended API access
+    :returns: :array: search results
+
+    """
+    
+    token = _getauthtoken(authtoken)
+    search_url = 'http://www.quandl.com/api/v1/datasets.json?query='
+    #parse query for proper API submission
+    parsedquery = re.sub(" ", "+", query)
+    parsedquery = re.sub("&", "+", parsedquery)
+    url = search_url + parsedquery
+    #Use authtoken if present
+    if token:
+        url += '&auth_token=' + token
+    #Add search source if given
+    if source:
+        url += '&source_code=' + source
+    #Page to be searched 
+    url += '&page=' + str(page)
+    text= urlopen(url).read()
+    data = json.loads(text)
+    try:
+        datasets = data['docs']
+    except TypeError:
+        raise TypeError("There are no matches for this search")
+    datalist = []
+    for i in range(len(datasets)):
+        temp_dict ={}
+        temp_dict['name'] = datasets[i]['name']
+        temp_dict['code'] = datasets[i]['source_code'] + '/' + datasets[i]['code']
+        temp_dict['desc'] = datasets[i]['description']
+        temp_dict['freq'] = datasets[i]['frequency']
+        temp_dict['colname'] = datasets[i]['column_names']
+        datalist.append(temp_dict)
+        if prints and i < 4:
+            print('{0:20}       :        {1:50}'.format('Name',temp_dict['name']))
+            print('{0:20}       :        {1:50}'.format('Quandl Code',temp_dict['code']))
+            print('{0:20}       :        {1:50}'.format('Description',temp_dict['desc']))
+            print('{0:20}       :        {1:50}'.format('Frequency',temp_dict['freq']))
+            print('{0:20}       :        {1:50}'.format('Column Names',temp_dict['colname']))
+            print('\n\n')
+    return datalist
+
+# returns None is date is None
 def _parse_dates(date):
-    if date is None: 
+    if date is None:
         return date
     if isinstance(date, datetime.datetime):
         return date.date().isoformat()
@@ -162,15 +214,12 @@ def _parse_dates(date):
     return date.date().isoformat()
 
 
-# Helper function for actually making API call and downloading the file
 def _download(url):
     dframe = pd.read_csv(url, index_col=0, parse_dates=True)
     return dframe
 
 
-# Helper function to make html push
 def _htmlpush(url, raw_params):
-    import json
     page = url
     params = urlencode(raw_params)
     request = Request(page, params)
